import numpy as np
import matplotlib.pyplot as plt


# neuron parameters
Vreset = -70.0  # mV
Vth = -55.0  # mV
Cm = 250.0  # pF
tref = 2.0  # ms
R = 0.04  # Ohms
Vrest = -70.0  # mV

# simulation parameters
simtime = 0.2  # s
sampling_time = 0.001  # s
samples = round(simtime / sampling_time)
t = np.arange(0, simtime, sampling_time)

# external input current (pA) to pre-synaptic neuron
# constant current
Iext = 400.0 * np.ones(samples)


# synaptic parameters
ts = 0.01  # s
delay = 0.01  # s
w = 1  # mA


# membrane potential
Vm = np.zeros(samples)  # mV
Vm[0] = Vreset

# spike recording
spikes = np.zeros(samples)


# simulation of presynaptic neuron
refcounter = 0
for i in range(1, samples):

    if refcounter > 0:
        Vm[i] = Vm[i-1]
        refcounter -= 1
    else:

        dVm = Iext[i] / Cm - (Vm[i-1] - Vrest) / (Cm*R)
        Vm[i] = Vm[i-1] + dVm  # * 1mV

    if Vm[i] > Vth:
        Vm[i] = Vreset
        spikes[i] = 1.0
        refcounter = tref / 1000 / sampling_time


# simulation of postsynaptic EPSC
I = np.zeros(samples)
for i in range(samples):
    if spikes[i] > 0.5:
        for k in range(i + int(delay/sampling_time), samples):
            tt = (k-i) * sampling_time
            I[k] = I[k] + w * tt / ts * np.exp(-tt/ts)


# plots
# presyn
_, (ax1, ax2) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [3, 1]})
ax1.plot(t, Vm)
ax1.set_xlim((0, simtime))
ax1.set_ylabel('presynaptic V_m (mV)')


# EPSC
ax2.plot(t, I)
ax2.set_xlim((0, simtime))
ax2.set_xlabel("time (s)")
ax2.set_ylabel('EPSC (mA)')

plt.show()
