import numpy as np
import matplotlib.pyplot as plt


# neuron parameters
Vreset = -70.0  # mV
Vth = -55.0  # mV
Cm = 250.0  # pF
tref = 2.0  # ms

# simulation parameters
simtime = 0.2  # s
sampling_time = 0.001  # s
samples = round(simtime / sampling_time)
t = np.arange(0, simtime, sampling_time)

# external input current (pA)
# constant current
Iext = 100.0 * np.ones(samples)

# single pulse
# Iext = np.zeros(samples);
# Iext[10:20] = 100;


# membrane potential
Vm = np.zeros(samples)  # mV
Vm[0] = Vreset

# spike recording
spikes = []


# simulation
refcounter = 0
for i in range(1, samples):

    if refcounter > 0:
        Vm[i] = Vm[i-1]
        refcounter -= 1
    else:

        dVm = Iext[i] / Cm
        Vm[i] = Vm[i-1] + dVm  # * 1mV

    if Vm[i] > Vth:
        Vm[i] = Vreset
        spikes.append(t[i])
        refcounter = tref / 1000 / sampling_time


# plots
_, (ax1, ax2) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [4, 1]})
ax1.plot(t, Vm)
ax1.plot(t, (Iext-np.min(Iext))/(np.max(Iext)-np.min(Iext))*(np.max(Vm)-np.min(Vm))+np.min(Vm));
ax1.set_xlim((0, simtime))
ax1.set_ylabel('V_m (mV)')


# raster plot
for sp in spikes:
    ax2.axvline(sp, 0.1, 0.9)
ax2.set_xlim((0, simtime))
ax2.set_ylim((0, 1))
ax2.yaxis.set_visible(False)
ax2.set_xlabel("time (s)")

plt.show()
